@@ -297,7 +297,7 @@ _create_xauth_file_for_user (const char  *username,
                 g_set_error (error,
                              GDM_DISPLAY_ERROR,
                              GDM_DISPLAY_ERROR_GETTING_USER_INFO,
-                             _("could not find user \"%s\" on system"),
+                             _("could not find user “%s” on system"),
                              username);
                 goto out;
 
@@ -438,8 +438,18 @@ _get_auth_info_for_display (GdmDisplayAccessFile *file,
         gdm_display_is_local (display, &is_local, NULL);
 
         if (is_local) {
-                *family = FamilyWild;
-                *address = g_strdup ("localhost");
+                /* We could just use FamilyWild here except xauth
+                 * (and by extension su and ssh) doesn't support it yet
+                 *
+                 * https://bugs.freedesktop.org/show_bug.cgi?id=43425
+                 */
+                char localhost[HOST_NAME_MAX + 1] = "";
+                *family = FamilyLocal;
+                if (gethostname (localhost, HOST_NAME_MAX) == 0) {
+                        *address = g_strdup (localhost);
+                } else {
+                        *address = g_strdup ("localhost");
+                }
         } else {
                 *family = FamilyWild;
                 gdm_display_get_remote_hostname (display, address, NULL);
@@ -533,6 +543,18 @@ gdm_display_access_file_add_display_with_cookie (GdmDisplayAccessFile  *file,
                 display_added = TRUE;
         }
 
+        /* If we wrote a FamilyLocal entry, we still want a FamilyWild
+         * entry, because it's more resiliant against hostname changes
+         *
+         */
+        if (auth_entry.family == FamilyLocal) {
+                auth_entry.family = FamilyWild;
+
+                if (XauWriteAuth (file->priv->fp, &auth_entry)
+                    && fflush (file->priv->fp) != EOF) {
+                        display_added = TRUE;
+                }
+        }
 
         g_free (auth_entry.address);
         g_free (auth_entry.number);
@@ -541,65 +563,6 @@ gdm_display_access_file_add_display_with_cookie (GdmDisplayAccessFile  *file,
         return display_added;
 }
 
-gboolean
-gdm_display_access_file_remove_display (GdmDisplayAccessFile  *file,
-                                        GdmDisplay            *display,
-                                        GError               **error)
-{
-        Xauth           *auth_entry;
-        unsigned short  family;
-        unsigned short  address_length;
-        char           *address;
-        unsigned short  number_length;
-        char           *number;
-        unsigned short  name_length;
-        char           *name;
-
-
-        g_return_val_if_fail (file != NULL, FALSE);
-        g_return_val_if_fail (file->priv->path != NULL, FALSE);
-
-        _get_auth_info_for_display (file, display,
-                                    &family,
-                                    &address_length,
-                                    &address,
-                                    &number_length,
-                                    &number,
-                                    &name_length,
-                                    &name);
-
-        auth_entry = XauGetAuthByAddr (family,
-                                       address_length,
-                                       address,
-                                       number_length,
-                                       number,
-                                       name_length,
-                                       name);
-        g_free (address);
-        g_free (number);
-        g_free (name);
-
-        if (auth_entry == NULL) {
-                g_set_error (error,
-                             GDM_DISPLAY_ACCESS_FILE_ERROR,
-                             GDM_DISPLAY_ACCESS_FILE_ERROR_FINDING_AUTH_ENTRY,
-                             "could not find authorization entry");
-                return FALSE;
-        }
-
-        XauDisposeAuth (auth_entry);
-
-        if (fflush (file->priv->fp) == EOF) {
-                g_set_error (error,
-                             G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
-                             "%s", g_strerror (errno));
-                return FALSE;
-        }
-
-        return TRUE;
-}
-
 void
 gdm_display_access_file_close (GdmDisplayAccessFile  *file)
 {
